package com.example.androidmp3;

import android.app.ListActivity;
import android.content.BroadcastReceiver;
import android.content.ContentUris;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.provider.MediaStore;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

public class AndroidMP3 extends ListActivity 
{
  public static final String ACTION_NEXT = 
      "com.example.androidmp3.action.NEXTTRACK";
  public static final String ACTION_GETTRACK = 
      "com.example.androidmp3.action.GETTRACK";
  public static final String TRACK_URI = "trackuri";

  private static final String MEDIA_PATH = new String("/sdcard/");
  private static final String TAG = "AndroidMP3";
  private List<MediaItem> mp3List = new ArrayList<MediaItem>();
  private List<String> mp3TextList = new ArrayList<String>();
  private Button playpauseButton;
  private Button nextButton;
  private Button prevButton;
  private int currentPosition = 0;

  /** Called when the activity is first created. */
  @Override
  public void onCreate(Bundle savedInstanceState)
  {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.list);
    setUpButtons();
    updateMP3List();
    IntentFilter filter = new IntentFilter();
    filter.addAction(ACTION_NEXT);
    filter.addAction(ACTION_GETTRACK);
    this.registerReceiver(this.broadcastReceiver, filter);
  }

  public void updateMP3List() {
    mp3List.clear();
    mp3TextList.clear();

    Uri uri = MediaStore.Audio.Media.EXTERNAL_CONTENT_URI;
    Cursor cur = managedQuery(uri, null, null, null, null);
    if (cur == null) {
      Log.e(TAG, "Music query failed; cursor null");
      return;
    }
    if (!cur.moveToFirst()) {
      Log.e(TAG, "No music found on SD card");
      return;
    }
    int titleCol  = cur.getColumnIndex(MediaStore.Audio.Media.TITLE);
    int artistCol = cur.getColumnIndex(MediaStore.Audio.Media.ARTIST);
    int idCol     = cur.getColumnIndex(MediaStore.Audio.Media._ID);
    int durCol    = cur.getColumnIndex(MediaStore.Audio.Media.DURATION);

    do {
      MediaItem item =  new MediaItem(cur.getLong(idCol), 
                                      cur.getString(titleCol), 
                                      cur.getString(artistCol), 
                                      cur.getLong(durCol));
      String itemText = item.getTitle() + " - " + item.getArtist();
      mp3List.add(item);
      mp3TextList.add(itemText);
    } while (cur.moveToNext());

    ArrayAdapter<String> mp3ListAdapter = 
        new ArrayAdapter<String>(this, R.layout.item, mp3TextList);
    setListAdapter(mp3ListAdapter);
  }

  @Override
  protected void onListItemClick(ListView list, View view, 
                                 int position, long id) {
    currentPosition = position;
    playpauseMp3(mp3List.get(position).getUri());
  }

  @Override
  protected void onPause() {
    super.onPause();
  }

  @Override
  protected void onResume() {
    super.onResume();
    updateMP3List();
    Intent intent = new Intent(ACTION_GETTRACK);
    startService(intent);
  }

  @Override
  protected void onDestroy() {
    super.onDestroy();
    unregisterReceiver(broadcastReceiver);
  }

  private BroadcastReceiver broadcastReceiver = new BroadcastReceiver() {
    @Override
    public void onReceive(Context c, Intent i) {
      if (i.getAction().equals(ACTION_NEXT)) {
          nextMp3();
      }
      else if (i.getAction().equals(ACTION_GETTRACK)) {
        setCurrentPosition(i.getStringExtra(TRACK_URI));
      }
    }
  };

  private void setUpButtons() {
    playpauseButton = (Button)this.findViewById(R.id.playpause);
    nextButton = (Button)this.findViewById(R.id.next);
    prevButton = (Button)this.findViewById(R.id.prev);

    playpauseButton.setOnClickListener(new OnClickListener() {
      @Override
      public void onClick(View v) {
        playpauseMp3(mp3List.get(currentPosition).getUri());
      }
    });

    nextButton.setOnClickListener(new OnClickListener() {
      @Override
      public void onClick(View v) {
        nextMp3();
      }
    });

    prevButton.setOnClickListener(new OnClickListener() {
      @Override
      public void onClick(View v) {
        prevMp3();
      }
    });
  }

  private void playpauseMp3(Uri mp3Uri) {
    Intent intent = new Intent(MP3Service.ACTION_PLAYPAUSE);
    intent.setData(mp3Uri);
    startService(intent);
    setTrackInfo(currentPosition);
   }

  private void nextMp3() {
    if (++currentPosition >= mp3List.size()) {
      // No more MP3s! Reset currentPosition to start of list
      currentPosition = 0;
    } else {
      playpauseMp3(mp3List.get(currentPosition).getUri());
    }
  }

  private void prevMp3() {
    if (--currentPosition < 0) {
      // No more MP3s! Reset currentPosition to start of list
      currentPosition = 0;
    } else {
      playpauseMp3(mp3List.get(currentPosition).getUri());
    }
  }

  private void setCurrentPosition(String uriString) {
    if (uriString == null) {
        currentPosition = 0;
        return;
    }
    Cursor cur = managedQuery(Uri.parse(uriString), null, null, null, null);
    if (cur.moveToFirst()) {
      int idCol = cur.getColumnIndex(MediaStore.Audio.Media._ID);
      currentPosition = cur.getInt(idCol) - 1;
    } else {
      currentPosition = 0;
    }
  }

  private void setTrackInfo(int position) {
    TextView currentmp3 = (TextView)this.findViewById(R.id.currentmp3);
    MediaItem item = mp3List.get(currentPosition);
    String title = item.getTitle();
    String artist = item.getArtist();
    long durationMillis = item.getDuration();

    long durationSecs = TimeUnit.SECONDS.convert(durationMillis, 
                                                 TimeUnit.MILLISECONDS);
    long durationMins = TimeUnit.MINUTES.convert(durationMillis, 
                                                 TimeUnit.MILLISECONDS);
    String duration = durationMins + ":" + (durationSecs-durationMins*60);
    currentmp3.setText(title + " - " + artist + " (" + duration + ")");
  }

  public class MediaItem {
    long id;
    String title; 
    String artist;
    long duration;

    public MediaItem(long id, String title, String artist, long duration) {
      this.id = id;
      this.title = title;
      this.artist = artist;
      this.duration = duration;
    }

    public long   getId()       { return id; }
    public String getTitle()    { return title; }
    public String getArtist()   { return artist; }
    public long   getDuration() { return duration; }
    public Uri    getUri()      {
      return ContentUris.withAppendedId(
                MediaStore.Audio.Media.EXTERNAL_CONTENT_URI, id);
    }
  }
}
