package com.example.androidmp3;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.media.MediaPlayer.OnErrorListener;
import android.net.Uri;
import android.os.Binder;
import android.os.IBinder;
import android.os.PowerManager;
import android.util.Log;
import android.widget.Toast;

public class MP3Service extends Service implements OnErrorListener, 
       AudioManager.OnAudioFocusChangeListener {
    
  private static final String TAG = "MP3Service";
  private final IBinder ib = new MP3Binder();
  private MediaPlayer mp;
  private Uri currentUri;
  private NotificationManager nm;
  private Notification notification = null;
  private int NOT_ID = 1;

  private enum State {
      Playing,
      Paused,
      Stopped
  }
  private State state = State.Stopped;

  public static final String ACTION_PLAYPAUSE = 
      "com.example.androidmp3.action.PLAYPAUSE";
  public static final String ACTION_PAUSE = 
      "com.example.androidmp3.action.PAUSE";
  public static final String ACTION_GETTRACK = 
      "com.example.androidmp3.action.GETTRACK";

  public class MP3Binder extends Binder {
    MP3Service getService() {
        return MP3Service.this;
    }
  }

  @Override
  public void onCreate() {
    mp = new MediaPlayer();
    mp.setOnErrorListener(this);
    mp.setWakeMode(getApplicationContext(), PowerManager.PARTIAL_WAKE_LOCK);
    nm = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
    notification = new Notification();
    notification.icon = R.drawable.icon;
  }

  @Override
  public int onStartCommand(Intent intent, int flags, int startId) {
    String action = intent.getAction();
    Uri path = intent.getData();
    if (action.equals(ACTION_PLAYPAUSE)) playpauseMp3(path);
    if (action.equals(ACTION_PAUSE)) pauseMp3();
    if (action.equals(ACTION_GETTRACK)) sendTrackUri();
    return START_NOT_STICKY;
  }

  @Override
  public void onDestroy() {
    nm.cancel(NOT_ID);
    mp.release();
  }

  public void onAudioFocusChange(int focusChange) {
    switch (focusChange) {
      case AudioManager.AUDIOFOCUS_GAIN:
        playpauseMp3(currentUri);
        break;
      case AudioManager.AUDIOFOCUS_LOSS:
      case AudioManager.AUDIOFOCUS_LOSS_TRANSIENT:
      case AudioManager.AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK:
        pauseMp3();
        break;
      default:
    }
  }

  @Override
  public IBinder onBind (Intent intent) {
        return ib;
  }

  @Override
  public boolean onError(MediaPlayer mp, int what, int extra) {
    Toast.makeText(getApplicationContext(), 
                   "AndroidMP3 error! Resetting media player.",
                   Toast.LENGTH_SHORT).show();
    Log.e(TAG, "Error: what=" + String.valueOf(what) + 
               ", extra=" + String.valueOf(extra));
    state = State.Stopped;
    return true; 
  }

  private void playpauseMp3(Uri uri) {
    try {
      if (getAudioFocus() == 1) { 
        Log.e(TAG, "can't get focus");
        return;
      }
      if ( (state == State.Playing) && (uri.equals(currentUri)) ) {
        pauseMp3();
      } else {
        if ( (state == State.Stopped) || (!uri.equals(currentUri)) ) {
          mp.reset();
          mp.setAudioStreamType(AudioManager.STREAM_MUSIC);
          mp.setDataSource(getApplicationContext(), uri);
          mp.prepare();
        } 
        mp.start();
        state = State.Playing;
        currentUri = uri;
        updateNotification("playing");
      }
    } catch (Exception e) {
      Log.e(TAG, "message is: " + e.getMessage());
      e.printStackTrace();
    }

    mp.setOnCompletionListener(new OnCompletionListener() {
      public void onCompletion(MediaPlayer mplayer) {
        sendBroadcast(new Intent(AndroidMP3.ACTION_NEXT));
      }
    });
  }

  private void pauseMp3() {
    try {
      mp.pause();
      state = State.Paused;
      updateNotification("paused");
    } catch (Exception e) {
      Log.e(TAG, "message is: " + e.getMessage());
      e.printStackTrace();
    }
  }

  private int getAudioFocus() {
    AudioManager am = (AudioManager) getSystemService(Context.AUDIO_SERVICE);
    int result = am.requestAudioFocus(this, AudioManager.STREAM_MUSIC, 
                                      AudioManager.AUDIOFOCUS_GAIN);
    if (result != AudioManager.AUDIOFOCUS_REQUEST_GRANTED) {
      // could not get audio focus, return error
      return 1;
    } else {
      return 0;
    }
  }
  
  private void updateNotification(String text) {
    Context c = getApplicationContext();
    PendingIntent pi = PendingIntent.getActivity(c, 0, 
                       new Intent(c, AndroidMP3.class),
                       PendingIntent.FLAG_UPDATE_CURRENT);
    notification.setLatestEventInfo(c, "AndroidMP3", text, pi);
    nm.notify(NOT_ID, notification);
    startForeground(NOT_ID, notification);
  }

  private void sendTrackUri() {
    Intent bcIntent = new Intent(AndroidMP3.ACTION_GETTRACK);
    if (currentUri != null) {
        bcIntent.putExtra(AndroidMP3.TRACK_URI, currentUri.toString()); 
    }
    sendBroadcast(bcIntent);
  }

}
